function [Y,X, fval,time] = biclustersdp_ADM(W,k,tol,maxiter, quiet)

% This code solves the following optimization problem for finding
% clustering
%
% max tr(W Z)
% s.t. Z_UU e <= e, Z_VV e <=e, Z >= 0
%      tr(Z) = 2k, Z is p.s.d.
%
% The method starts by reformulating the above problem as
%
% max tr(W Y)
% s.t. X_UU e <= e, X_VV e <=e, X >= 0
%      tr(Y) = 2k, Y is p.s.d.
%      X - Y = 0
%
% The alternating direction method of multiplier is then applied to the
% above problem. 
% % The minimization with respect to Y variable has "closed-form"
% % solution, while the X variable is solved approximately via the spectral projected gradient method.
% % The method is terminated by checking relative duality gap and absolute
% % primal feasibility w.r.t. the first two inequalities in the above
% % formulation.
%
% Input:
%
% W       - symmetric matrix containing edge weight
% k       - number of clusters
% tol     - termination tolerance
% maxiter - maximum number of iterations
%
% Output:
%
% Y       - approximate optimal solution matrix containing clustering
%           information
% fval    - primal function value at termination
% time    - cpu time
%
% Written by TBA
% March 11, 2012
%
% Notes: works ok for 5 <= k <= 15, average cluster size around 100, convergence
% is bad when cluster sizes are small

tic

if quiet == 0
    fprintf(' ********* iteration starts *********\n')
end

% input size
[m,n] = size(W);

% parameters
W = [zeros(m), -W; -W', zeros(n)]; % the following is a minimizer algorithm
beta = min(max(5*n/k,80),500);
if quiet == 0
    fprintf(' penalty used %g\n', beta)
end

iter = 0;
em = ones(m,1);
en = ones(n,1);
N = m+n;

% initialization
Y = zeros(N,N);
X = Y;
U = X;
lambda = ones(m,1);
phi = ones(n,1);

% PROPACK settings.
%OPTIONS.cgs = 1;


while iter < maxiter
  
  % update Y
  tmp = X - (W + U)/beta;
  [V,S] = eig(tmp);
  %[V,S] = laneig(tmp, N, 'AL',OPTIONS);
  d = simplex_proj(diag(S),2*k);
  Y = V*spdiags(d,0,N,N)/V;
  Y = (Y + Y')/2;
  
  % update X
  [X,lambda, phi] = bicluster_update_X(m,n,Y,U,beta,tol/10,lambda, phi);
  
  % update U
  U = U - beta*(X - Y);
  U = (U+U')/2;
  
  % check for termination
  minY = min(min(Y));
  minY_uu = min(em - Y(1:m,1:m)*em);
  minY_vv = min(en - Y(m+1:N, m+1:N)*en);
  %size(minY), size(minY_uu), size(minY_vv)
  pfeas = min([minY; minY_uu; minY_vv]); % absolute primal feasibility
  fval = sum(sum(W.*Y));
  %M1 = W(1:m,1:m)+U(1:m,1:m);
  %M2 = W(m+1:N,m+1:N) + U(m+1:N,m+1:N) ;
  dval = 2*k*min(eig(W+U)) - sum(sum(X.*U)); % asymptotically converging to dual optimal value, not necessarily a lower bound.
  %dval = k*(min(eig(M1)) + min(eig(M2))) - sum(sum(X.*U)); % asymptotically converging to dual optimal value, not necessarily a lower bound.
  relgap = abs(fval - dval)/max(abs(fval),1); % relative duality gap
  if mod(iter,2) == 0 && iter > 0    && quiet ==0
    fprintf(' iter %4d   fval %10g   dval %10g   pfeas %10g\n', iter, -fval, -dval, pfeas)
  end
  if relgap < tol && pfeas > -tol
    break
  end
  
  iter = iter + 1;
  
end

if quiet == 0
    fprintf(' ********** termination ************\n')
    fprintf(' iter %4d   fval %10g   dval %10g   pfeas %10g\n', iter, -fval, -dval, pfeas)
end
fval= -fval;
time = toc;

if quiet == 0
    fprintf(' time elapsed %g\n', time)
end


X = Y(1:m, (m+1):m+n);