function [Y,fval,time] = clustersdp_ADM(W,k,tol,maxiter, quiet)

% This code solves the following optimization problem for finding
% clustering
%
% max tr(W X)
% s.t. X e <= e, X >= 0
%      tr(X) = k, X is p.s.d.
%
% The method starts by reformulating the above problem as
%
% max tr(W Y)
% s.t. X e <= e, X >= 0
%      tr(Y) = k, Y is p.s.d.
%      X - Y = 0
%
% The alternating direction method of multiplier is then applied to the
% above problem. 
% % The minimization with respect to Y variable has "closed-form"
% % solution, while the X variable is solved approximately via the spectral projected gradient method.
% % The method is terminated by checking "relative duality gap" and absolute
% % primal feasibility w.r.t. the first two inequalities in the above
% % formulation.
%
% Input:
%
% W       - symmetric matrix containing edge weight
% k       - number of clusters
% tol     - termination tolerance
% maxiter - maximum number of iterations
%
% Output:
%
% Y       - approximate optimal solution matrix containing clustering
%           information
% fval    - primal function value at termination
% time    - cpu time
%
% Written by TBA
% March 21, 2012

tic

% input size
n = size(W,1);

% parameters
W = -W; % the following is a minimizer algorithm
beta = min(max(5*n/k,80),500)/2;
% beta = min(max(5*n/k,80),1000); % in the presence of points not in any clusters
if quiet ==0
    fprintf(' ********* iteration starts *********\n')
    fprintf(' penalty used %g\n', beta)
end
iter = 0;
e = ones(n,1);

% initialization
Y = zeros(n,n);
X = Y;
U = X;
lambda = ones(n,1);

while iter < maxiter
  
  % update Y
  tmp = X - (W + U)/beta;
  [V,S] = eig(tmp);
  d = simplex_proj(diag(S),k);
  Y = V*spdiags(d,0,n,n)/V;
  Y = (Y + Y')/2;
  
  % update X
  [X,lambda] = cluster_update_X(n,Y,U,beta,tol/10,lambda);
  
  % update U
  U = U - beta*(X - Y);
  
  % check for termination
  pfeas = min(min(min(Y)),min(e - Y*e)); % absolute primal feasibility
  fval = sum(sum(W.*Y));
  dval = k*min(eig(W + U)) - sum(sum(X.*U)); % asymptotically converging to dual optimal value, not necessarily a lower bound.
  relgap = abs(fval - dval)/max(abs(fval),1); % relative duality gap

  if quiet == 0
      if mod(iter,2) == 0 && iter > 0
          fprintf(' iter %4d   fval %10g   dval %10g   pfeas %10g\n', iter, -fval, -dval, pfeas)
      end
  end
  
  if relgap < tol && pfeas > -tol 
    break
  end
  
  iter = iter + 1;
  
end

if quiet ==0
    fprintf(' ********** termination ************\n')
    fprintf(' iter %4d   fval %10g   dval %10g   pfeas %10g\n', iter, -fval, -dval, pfeas)
    imagesc(Y)
end
fval= -fval;
time = toc;

if quiet ==0
    fprintf(' time elapsed %g\n', time)
end